/*
 * Copyright 2012 Decebal Suiu
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this work except in compliance with
 * the License. You may obtain a copy of the License in the LICENSE file, or at:
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */
package ro.fortsoft.pf4j;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 * Represents the version of a Plugin and allows versions to be compared.
 * Version identifiers have five components.
 *
 *  1. Major version. A non-negative integer.
 *  2. Minor version. A non-negative integer.
 *  3. Release version. A non-negative integer.
 *  4. Build version. A non-negative integer.
 *  5. Qualifier. A text string.
 *
 * This class is immutable.
 *
 * @author Decebal Suiu
 */
public class PluginVersion implements Comparable<PluginVersion> {

	private int major;
	private int minor;
	private int release;
	private int build;
	private String qualifier;

    private PluginVersion() {
    }

	public PluginVersion(int major, int minor, int release) {
		this.major = major;
		this.minor = minor;
		this.release = release;
	}

	public PluginVersion(int major, int minor, int release, int build) {
		this.major = major;
		this.minor = minor;
		this.release = release;
		this.build = build;
	}

	public PluginVersion(int major, int minor, int release, int build, String qualifier) {
		this.major = major;
		this.minor = minor;
		this.release = release;
		this.build = build;
		this.qualifier = qualifier;
	}

	public static PluginVersion createVersion(String version) {
        if (version == null) {
            return new PluginVersion();
        }

		PluginVersion v = new PluginVersion();

		StringTokenizer st = new StringTokenizer(version, ".");
		List<String> tmp = new ArrayList<String>();
		for (int i = 0; st.hasMoreTokens() && i < 4; i++) {
			tmp.add(st.nextToken());
		}

		int n = tmp.size();
		switch (n) {
			case 0 :
				break;
			case 1 :
				v.major = Integer.parseInt(tmp.get(0));
				break;
			case 2 :
				v.major = Integer.parseInt(tmp.get(0));
				v.minor = Integer.parseInt(tmp.get(1));
				break;
			case 3 :
				v.major = Integer.parseInt(tmp.get(0));
				v.minor = Integer.parseInt(tmp.get(1));
				v.release = Integer.parseInt(tmp.get(2));
				break;
			case 4 :
				v.major = Integer.parseInt(tmp.get(0));
				v.minor = Integer.parseInt(tmp.get(1));
				v.release = Integer.parseInt(tmp.get(2));
				v.build = Integer.parseInt(tmp.get(3));
				break;
		}

		return v;
	}

	public int getMajor() {
		return this.major;
	}

	public int getMinor() {
		return this.minor;
	}

	public int getRelease() {
		return this.release;
	}

    public int getBuild() {
        return this.build;
    }

    public String getQualifier() {
		return qualifier;
	}

	public String toString() {
        StringBuffer sb = new StringBuffer(50);
        sb.append(major);
        sb.append('.');
        sb.append(minor);
        sb.append('.');
        sb.append(release);
        sb.append('.');
        sb.append(build);
        if (qualifier != null) {
        	sb.append(qualifier);
        }

        return sb.toString();
    }

    public int compareTo(PluginVersion version) {
        if (version.major > major) {
            return 1;
        } else if (version.major < major) {
            return -1;
        }

        if (version.minor > minor) {
            return 1;
        } else if (version.minor < minor) {
            return -1;
        }

        if (version.release > release) {
            return 1;
        } else if (version.release < release) {
            return -1;
        }

        if (version.build > build) {
            return 1;
        } else if (version.build < build) {
            return -1;
        }

        return 0;
    }

    /*
    private String extractQualifier(String token) {
    	StringTokenizer st = new StringTokenizer(token, "-");
    	if (st.countTokens() == 2) {
    		return st.
    	}
    }
    */

    // for test only
    public static void main(String[] args) {
        PluginVersion v = PluginVersion.createVersion("4.0.0.123");
        System.out.println(v.toString());
//        v = PluginVersion.createVersion("4.0.0.123-alpha");
//        System.out.println(v.toString());
        PluginVersion v1 = PluginVersion.createVersion("4.1.0");
        System.out.println(v1.toString());
        PluginVersion v2  = PluginVersion.createVersion("4.0.32");
        System.out.println(v2.toString());
        System.out.println(v1.compareTo(v2));
    }

}
